########################################################################################################################
# Compute differences between two existing check constraint
########################################################################################################################

require_relative '../model/check_constraint'
require_relative 'comparison_error'
require_relative 'diff'

module Dbsketch
	module Comparison

		class CheckConstraintDiff < Diff
			def initialize old_constraint, new_constraint
				super old_constraint, new_constraint
			end
			alias :old_constraint :old_value
			alias :new_constraint :new_value

			def condition
				### Preconditions
				raise ComparisonError, "no change found" if not change?
				###
				Diff.new old_constraint.condition, new_constraint.condition
			end
		end

	end
end

module Dbsketch
	module Comparison

		class CheckConstraintComparator

			def are_equivalent? old_constraint, new_constraint
				### Preconditions
				raise ArgumentError, "old_constraint is not a Dbsketch::Model::CheckConstraint" unless nil == old_constraint or old_constraint.is_a? Dbsketch::Model::CheckConstraint
				raise ArgumentError, "new_constraint is not a Dbsketch::Model::CheckConstraint" unless nil == new_constraint or new_constraint.is_a? Dbsketch::Model::CheckConstraint
				###
				(nil != old_constraint and nil != new_constraint) and (old_constraint.condition == new_constraint.condition)
			end

			def compare old_constraint, new_constraint
				### Preconditions
				raise ArgumentError, "old_constraint is not a Dbsketch::Model::CheckConstraint" unless nil == old_constraint or old_constraint.is_a? Dbsketch::Model::CheckConstraint
				raise ArgumentError, "new_constraint is not a Dbsketch::Model::CheckConstraint" unless nil == new_constraint or new_constraint.is_a? Dbsketch::Model::CheckConstraint
				###
				CheckConstraintDiff.new(old_constraint, new_constraint) if not are_equivalent? old_constraint, new_constraint
			end

		end

	end
end
