########################################################################################################################
# Compute differences between two existing columns
########################################################################################################################

require_relative '../model/column'
require_relative 'diff'
require_relative 'type_comparator'

module Dbsketch
	module Comparison

		class ColumnDiff < Diff
			def initialize old_column, new_column, type_comparator, options
				super old_column, new_column
				if change?
					@type = type_comparator.compare(old_column.type, new_column.type)
					@identity = Diff.new(old_column.identity, new_column.identity) if old_column.identity != new_column.identity
					@default = Diff.new(old_column.default, new_column.default) if old_column.default != new_column.default
					@nullable = Diff.new(old_column.nullable, new_column.nullable) if old_column.nullable != new_column.nullable
					@order = Diff.new(old_column.order, new_column.order) if options[:compare_order] and old_column.order != new_column.order
				end
			end
			
			attr_reader :type, :identity, :default, :nullable, :order
			alias :old_column :old_value
			alias :new_column :new_value
		end

	end
end

module Dbsketch
	module Comparison

		class ColumnComparator
			def initialize options: {}
				@options = { :compare_order => true }.merge options
				@type_comparator = TypeComparator.new
			end

			def are_equivalent? old_column, new_column
				### Preconditions
				raise ArgumentError, "old_column is not a Dbsketch::Model::Column" unless nil == old_column or old_column.is_a? Dbsketch::Model::Column
				raise ArgumentError, "new_column is not a Dbsketch::Model::Column" unless nil == new_column or new_column.is_a? Dbsketch::Model::Column
				###
				(nil != old_column and nil != new_column) and
					@type_comparator.are_equivalent?(old_column.type, new_column.type) and
					old_column.identity == new_column.identity and
					old_column.default == new_column.default and
					old_column.nullable == new_column.nullable and
					(not @options[:compare_order] or old_column.order == new_column.order)
			end

			def compare old_column, new_column
				### Preconditions
				raise ArgumentError, "old_column is not a Dbsketch::Model::Column" unless nil == old_column or old_column.is_a? Dbsketch::Model::Column
				raise ArgumentError, "new_column is not a Dbsketch::Model::Column" unless nil == new_column or new_column.is_a? Dbsketch::Model::Column
				###
				ColumnDiff.new(old_column, new_column, @type_comparator, @options) if not are_equivalent? old_column, new_column
			end

		end

	end
end
