########################################################################################################################
# Compute differences between two existing computed columns
########################################################################################################################

require_relative '../model/computed_column'
require_relative 'diff'
require_relative 'type_comparator'

module Dbsketch
	module Comparison

		class ComputedColumnDiff < Diff
			def initialize old_column, new_column, options
				super old_column, new_column
				if change?
					@query = Diff.new(old_column.query, new_column.query) if old_column.query != new_column.query
					@nullable = Diff.new(old_column.nullable, new_column.nullable) if old_column.nullable != new_column.nullable
					@order = Diff.new(old_column.order, new_column.order) if options[:compare_order] and old_column.order != new_column.order
					@persisted = Diff.new(old_column.persisted, new_column.persisted) if old_column.persisted != new_column.persisted
				end
			end

			attr_reader :query, :nullable, :order, :persisted
			alias :old_column :old_value
			alias :new_column :new_value
		end

	end
end

module Dbsketch
	module Comparison

		class ComputedColumnComparator

			def initialize options: {}
				@options = { :compare_order => true }.merge options
			end

			def are_equivalent? old_column, new_column
				### Preconditions
				raise ArgumentError, "old_column is not a Dbsketch::Model::ComputedColumn" unless nil == old_column or old_column.is_a? Dbsketch::Model::ComputedColumn
				raise ArgumentError, "new_column is not a Dbsketch::Model::ComputedColumn" unless nil == new_column or new_column.is_a? Dbsketch::Model::ComputedColumn
				###
				(nil != old_column and nil != new_column) and
					old_column.query == new_column.query and
					old_column.nullable == new_column.nullable and
					old_column.persisted == new_column.persisted and
					(not @options[:compare_order] or old_column.order == new_column.order)
			end

			def compare old_column, new_column
				### Preconditions
				raise ArgumentError, "old_column is not a Dbsketch::Model::ComputedColumn" unless nil == old_column or old_column.is_a? Dbsketch::Model::ComputedColumn
				raise ArgumentError, "new_column is not a Dbsketch::Model::ComputedColumn" unless nil == new_column or new_column.is_a? Dbsketch::Model::ComputedColumn
				###
				ComputedColumnDiff.new(old_column, new_column, @options) if not are_equivalent? old_column, new_column
			end

		end

	end
end
