########################################################################################################################
# Compute differences between two existing foreign keys
########################################################################################################################

require_relative '../model/foreign_key'
require_relative 'diff'

module Dbsketch
	module Comparison

		class ForeignKeyDiff < Diff
			def initialize old_key, new_key
				super old_key, new_key
				if change?
					@constricted_column = Diff.new(old_key.constricted_column.name, new_key.constricted_column.name) if old_key.constricted_column.name != new_key.constricted_column.name
					@referenced_table = Diff.new(old_key.referenced_table.name, new_key.referenced_table.name) if old_key.referenced_table.name != new_key.referenced_table.name
					@referenced_column = Diff.new(old_key.referenced_column.name, new_key.referenced_column.name) if old_key.referenced_column.name != new_key.referenced_column.name
				end
			end

			attr_reader :constricted_column, :referenced_table, :referenced_column
			alias :old_key :old_value
			alias :new_key :new_value
		end

	end
end

module Dbsketch
	module Comparison

		class ForeignKeyComparator

			def are_equivalent? old_key, new_key
				### Preconditions
				raise ArgumentError, "old_key is not a Dbsketch::Model::ForeignKey" unless nil == old_key or old_key.is_a? Dbsketch::Model::ForeignKey
				raise ArgumentError, "new_key is not a Dbsketch::Model::ForeignKey" unless nil == new_key or new_key.is_a? Dbsketch::Model::ForeignKey
				###
				(nil != old_key and nil != new_key) and
					old_key.constricted_column.name == new_key.constricted_column.name and
					old_key.referenced_table.name == new_key.referenced_table.name and
					old_key.referenced_column.name == new_key.referenced_column.name
			end

			def compare old_key, new_key
				### Preconditions
				raise ArgumentError, "old_key is not a Dbsketch::Model::ForeignKey" unless nil == old_key or old_key.is_a? Dbsketch::Model::ForeignKey
				raise ArgumentError, "new_key is not a Dbsketch::Model::ForeignKey" unless nil == new_key or new_key.is_a? Dbsketch::Model::ForeignKey
				###
				ForeignKeyDiff.new(old_key, new_key) if not are_equivalent? old_key, new_key
			end

		end

	end
end
