########################################################################################################################
# Compute differences between two functions
########################################################################################################################

require_relative '../model/function'
require_relative 'diff'

module Dbsketch
	module Comparison

		class FunctionDiff < Diff
			def initialize old_function, new_function
				super old_function, new_function
				if change?
					@arguments = Diff.new(old_function.arguments, new_function.arguments) if old_function.arguments != new_function.arguments
					@returns = Diff.new(old_function.returns, new_function.returns) if old_function.returns != new_function.returns
					@algo = Diff.new(old_function.algo, new_function.algo) if old_function.algo != new_function.algo
				end
			end

			attr_reader :arguments, :returns, :algo
			alias :old_function :old_value
			alias :new_function :new_value
		end

	end
end

module Dbsketch
	module Comparison

		class FunctionComparator

			def are_equivalent? old_function, new_function
				### Preconditions
				raise ArgumentError, "old_function is not a Dbsketch::Model::Function" unless nil == old_function or old_function.is_a? Dbsketch::Model::Function
				raise ArgumentError, "new_function is not a Dbsketch::Model::Function" unless nil == new_function or new_function.is_a? Dbsketch::Model::Function
				###
				(nil != old_function and nil != new_function) and
					old_function.arguments == new_function.arguments and
					old_function.returns == new_function.returns and
					old_function.algo == new_function.algo
			end

			def compare old_function, new_function
				### Preconditions
				raise ArgumentError, "old_function is not a Dbsketch::Model::Function" unless nil == old_function or old_function.is_a? Dbsketch::Model::Function
				raise ArgumentError, "new_function is not a Dbsketch::Model::Function" unless nil == new_function or new_function.is_a? Dbsketch::Model::Function
				###
				FunctionDiff.new(old_function, new_function) if not are_equivalent? old_function, new_function
			end

		end

	end
end
