########################################################################################################################
# Database differences between two primary keys
########################################################################################################################

require_relative '../model/primary_key'
require_relative 'diff'

module Dbsketch
	module Comparison

		class PrimaryKeyDiff < Diff
			def initialize old_key, new_key, columns
				super old_key, new_key
				### Preconditions
				columns.each_with_index { |column, index| raise ArgumentError, "columns[#{index}] is not a Dbsketch::Comparison::Diff" unless column.is_a? Dbsketch::Comparison::Diff }
				###
				if change?
					@name = Diff.new(old_key.name, new_key.name) if old_key.name != new_key.name
				end
				@columns = columns
			end

			attr_reader :name, :columns
			alias :old_key :old_value
			alias :new_key :new_value
		end

		class PrimaryKeyComparator

			def are_equivalent? old_key, new_key
				### Preconditions
				raise ArgumentError, "old_key is not a Dbsketch::Model::PrimaryKey" unless nil == old_key or old_key.is_a? Dbsketch::Model::PrimaryKey
				raise ArgumentError, "new_key is not a Dbsketch::Model::PrimaryKey" unless nil == new_key or new_key.is_a? Dbsketch::Model::PrimaryKey
				###
				(nil == old_key and nil == new_key) or (nil != old_key and nil != new_key and old_key.name == new_key.name and columns(old_key, new_key).empty?)
			end

			# Returns a PrimaryKeyDiff if tables are different, nil otherwise
			def compare old_key, new_key
				### Preconditions
				raise ArgumentError, "old_key is not a Dbsketch::Model::PrimaryKey" unless nil == old_key or old_key.is_a? Dbsketch::Model::PrimaryKey
				raise ArgumentError, "new_key is not a Dbsketch::Model::PrimaryKey" unless nil == new_key or new_key.is_a? Dbsketch::Model::PrimaryKey
				###
				PrimaryKeyDiff.new(old_key, new_key, columns(old_key, new_key)) if not are_equivalent? old_key, new_key
			end

			private

			def columns old_key, new_key
				diffs = []
				if nil != old_key and nil != new_key
					diffs << old_key.columns.select { |old_k| not new_key.has_column? old_k.name }.map { |old_k| Diff.new old_k.name, nil }
					diffs << new_key.columns.select { |new_k| not old_key.has_column? new_k.name }.map { |new_k| Diff.new nil, new_k.name }
				end
				diffs.flatten.compact
			end

		end

	end
end
