########################################################################################################################
# Compute differences between two procedures
########################################################################################################################

require_relative '../model/procedure'
require_relative 'diff'

module Dbsketch
	module Comparison

		class ProcedureDiff < Diff
			def initialize old_procedure, new_procedure
				super old_procedure, new_procedure
				if change?
					@arguments = Diff.new(old_procedure.arguments, new_procedure.arguments) if old_procedure.arguments != new_procedure.arguments
					@algo = Diff.new(old_procedure.algo, new_procedure.algo) if old_procedure.algo != new_procedure.algo
				end
			end

			attr_reader :arguments, :algo
			alias :old_procedure :old_value
			alias :new_procedure :new_value
		end

	end
end

module Dbsketch
	module Comparison

		class ProcedureComparator

			def are_equivalent? old_procedure, new_procedure
				### Preconditions
				raise ArgumentError, "old_procedure is not a Dbsketch::Model::Procedure" unless nil == old_procedure or old_procedure.is_a? Dbsketch::Model::Procedure
				raise ArgumentError, "new_procedure is not a Dbsketch::Model::Procedure" unless nil == new_procedure or new_procedure.is_a? Dbsketch::Model::Procedure
				###
				(nil != old_procedure and nil != new_procedure) and
					old_procedure.arguments == new_procedure.arguments and
					old_procedure.algo == new_procedure.algo
			end

			def compare old_procedure, new_procedure
				### Preconditions
				raise ArgumentError, "old_procedure is not a Dbsketch::Model::Procedure" unless nil == old_procedure or old_procedure.is_a? Dbsketch::Model::Procedure
				raise ArgumentError, "new_procedure is not a Dbsketch::Model::Procedure" unless nil == new_procedure or new_procedure.is_a? Dbsketch::Model::Procedure
				###
				ProcedureDiff.new(old_procedure, new_procedure) if not are_equivalent? old_procedure, new_procedure
			end

		end

	end
end
