########################################################################################################################
# Compute differences between two triggers
########################################################################################################################

require_relative '../model/trigger'
require_relative 'comparison_error'
require_relative 'diff'

module Dbsketch
	module Comparison

		class TriggerDiff < Diff
			def initialize old_trigger, new_trigger
				super old_trigger, new_trigger
				if change?
					@target = Diff.new(old_trigger.target.name, new_trigger.target.name) if old_trigger.target.name != new_trigger.target.name
					@activation_time = Diff.new(old_trigger.activation_time, new_trigger.activation_time) if old_trigger.activation_time != new_trigger.activation_time
					@algo = Diff.new(old_trigger.algo, new_trigger.algo) if old_trigger.algo != new_trigger.algo
				end
			end
			attr_reader :target, :activation_time, :algo
			alias :old_trigger :old_value
			alias :new_trigger :new_value
		end

	end
end

module Dbsketch
	module Comparison

		class TriggerComparator

			def are_equivalent? old_trigger, new_trigger
				### Preconditions
				raise ArgumentError, "old_trigger is not a Dbsketch::Model::Trigger" unless nil == old_trigger or old_trigger.is_a? Dbsketch::Model::Trigger
				raise ArgumentError, "new_trigger is not a Dbsketch::Model::Trigger" unless nil == new_trigger or new_trigger.is_a? Dbsketch::Model::Trigger
				###
				(nil != old_trigger and nil != new_trigger) and (old_trigger.target.name == new_trigger.target.name) and (old_trigger.activation_time == new_trigger.activation_time) and (old_trigger.algo == new_trigger.algo)
			end

			def compare old_trigger, new_trigger
				### Preconditions
				raise ArgumentError, "old_trigger is not a Dbsketch::Model::Trigger" unless nil == old_trigger or old_trigger.is_a? Dbsketch::Model::Trigger
				raise ArgumentError, "new_trigger is not a Dbsketch::Model::Trigger" unless nil == new_trigger or new_trigger.is_a? Dbsketch::Model::Trigger
				###
				TriggerDiff.new(old_trigger, new_trigger) if not are_equivalent? old_trigger, new_trigger
			end

		end

	end
end
