########################################################################################################################
# Compute differences between two types
########################################################################################################################

require_relative '../model/type'
require_relative 'diff'

module Dbsketch
	module Comparison

		class TypeDiff < Diff
			def initialize old_type, new_type
				### Preconditions
				raise ArgumentError, "old_type is not a Dbsketch::Model::Type" unless old_type.is_a? Dbsketch::Model::Type
				raise ArgumentError, "new_type is not a Dbsketch::Model::Type" unless new_type.is_a? Dbsketch::Model::Type
				###
				super old_type, new_type
				@sql_type = Diff.new(old_type.sql_type, new_type.sql_type) if (old_type.sql_type != new_type.sql_type)
				@sizes = Diff.new(old_type.sizes, new_type.sizes) if (old_type.sizes != new_type.sizes)
			end
			attr_reader :sql_type, :sizes
		end

	end
end

module Dbsketch
	module Comparison

		class TypeComparator

			def are_equivalent? old_type, new_type
				### Preconditions
				raise ArgumentError, "old_type is not a Dbsketch::Model::Type" unless old_type.is_a? Dbsketch::Model::Type
				raise ArgumentError, "new_type is not a Dbsketch::Model::Type" unless new_type.is_a? Dbsketch::Model::Type
				###
				(old_type.sql_type == new_type.sql_type) and (old_type.sizes == new_type.sizes)
			end

			def compare old_type, new_type
				### Preconditions
				raise ArgumentError, "old_type is not a Dbsketch::Model::Type" unless old_type.is_a? Dbsketch::Model::Type
				raise ArgumentError, "new_type is not a Dbsketch::Model::Type" unless new_type.is_a? Dbsketch::Model::Type
				###
				TypeDiff.new(old_type, new_type) if not are_equivalent? old_type, new_type
			end

		end

	end
end
