########################################################################################################################
# Database differences between two existing unique constraint
########################################################################################################################

require_relative '../model/unique_constraint'
require_relative 'diff'

module Dbsketch
	module Comparison

		class UniqueConstraintDiff < Diff
			def initialize old_constraint, new_constraint, columns
				super old_constraint, new_constraint
				### Preconditions
				columns.each_with_index { |column, index| raise ArgumentError, "columns[#{index}] is not a Dbsketch::Comparison::Diff" unless column.is_a? Dbsketch::Comparison::Diff }
				###
				@columns = columns
			end

			attr_reader :columns
			alias :old_constraint :old_value
			alias :new_constraint :new_value
		end

		class UniqueConstraintComparator

			def are_equivalent? old_constraint, new_constraint
				### Preconditions
				raise ArgumentError, "old_constraint is not a Dbsketch::Model::UniqueConstraint" unless nil == old_constraint or old_constraint.is_a? Dbsketch::Model::UniqueConstraint
				raise ArgumentError, "new_constraint is not a Dbsketch::Model::UniqueConstraint" unless nil == new_constraint or new_constraint.is_a? Dbsketch::Model::UniqueConstraint
				###
				(nil != old_constraint and nil != new_constraint) and columns(old_constraint, new_constraint).empty?
			end

			# Returns a UniqueConstraintDiff if tables are different, nil otherwise
			def compare old_constraint, new_constraint
				### Preconditions
				raise ArgumentError, "old_constraint is not a Dbsketch::Model::UniqueConstraint" unless nil == old_constraint or old_constraint.is_a? Dbsketch::Model::UniqueConstraint
				raise ArgumentError, "new_constraint is not a Dbsketch::Model::UniqueConstraint" unless nil == new_constraint or new_constraint.is_a? Dbsketch::Model::UniqueConstraint
				###
				UniqueConstraintDiff.new(old_constraint, new_constraint, columns(old_constraint, new_constraint)) if not are_equivalent? old_constraint, new_constraint
			end

			private

			def columns old_constraint, new_constraint
				diffs = []
				if nil != old_constraint and nil != new_constraint
					diffs << old_constraint.columns.select { |old_c| not new_constraint.has_column? old_c.name }.map { |old_c| Diff.new old_c.name, nil }
					diffs << new_constraint.columns.select { |new_c| not old_constraint.has_column? new_c.name }.map { |new_c| Diff.new nil, new_c.name }
				end
				diffs.flatten.compact
			end

		end

	end
end
