########################################################################################################################
# Defines database column
########################################################################################################################

require_relative 'abstract_column'
require_relative 'type'

module Dbsketch
	module Model

		class Column < AbstractColumn
			def initialize name, type, meaning: nil, comment: nil, nullable: true, order: nil, identity: false, default: nil
				super(name, :meaning => meaning, :comment => comment, :nullable => nullable, :order => order)
				### Preconditions
				raise ArgumentError, "type is not a Dbsketch::Model::Type" unless type.is_a? Dbsketch::Model::Type
				raise ArgumentError, "identity is not a boolean" unless identity.is_a? TrueClass or identity.is_a? FalseClass
				###
				@type = type
				@identity = identity
				@default = default
			end

			attr_reader :type, :identity, :default
			attr_writer :default

			def compatible_with? other_column
				### Preconditions
				raise ArgumentError, "other_column is not a Dbsketch::Model::Column" unless other_column.is_a? Column
				###
				(@type.compatible_with? other_column.type) and (not @nullable or other_column.nullable)
			end
		end

	end
end
