########################################################################################################################
# Database representation
########################################################################################################################

require_relative 'custom_code'
require_relative 'index'
require_relative 'model_error'
require_relative 'operation'
require_relative 'table'
require_relative 'trigger'
require_relative 'view'

module Dbsketch
	module Model

		class Database
			def initialize
				@items = []
			end

			attr_reader :items

			def add object
				objects = object.is_a?(Array) ? object : [object]
				### Preconditions
				objects.each_with_index do |o, index|
					raise ArgumentError, "object n°#{index + 1} is not a Dbsketch::Model::CustomCode, Index, Operation, Table nor View" unless o.is_a? CustomCode or o.is_a? Index or o.is_a? Operation or o.is_a? Table or o.is_a? Trigger or o.is_a? View
					raise ModelError, "object #{o.name} already exists" if nil != (@items.find { |i| i.name.downcase == o.name.downcase })
				end
				###
				@items = @items + objects
			end

			def get_by_class given_class
				@items.select { |i| i.is_a? given_class }
			end

			def custom_code
				get_by_class CustomCode
			end

			def indexes
				get_by_class Index
			end

			def operations
				get_by_class Operation
			end

			def tables
				get_by_class Table
			end

			def triggers
				get_by_class Trigger
			end

			def views
				get_by_class View
			end

			def has_item? item_name
				### Preconditions
				raise ArgumentError, "item_name is not a String" unless item_name.is_a? String
				###
				nil != @items.find { |t| t.name.downcase == item_name.downcase }
			end

			def [] item_name
				### Preconditions
				raise ArgumentError, "item_name is not a String" unless item_name.is_a? String
				###
				item = @items.find { |t| t.name.downcase == item_name.downcase }
				raise ModelError, "item #{item_name} not found in database" if nil == item
				item
			end

			def order_items!
				@items.map! { |i| i.reset_order!; i }
				@items.map! { |i| i.compute_order!; i }
				@items.sort! { |a,b| (a.order != b.order) ? (a.order <=> b.order) : (a.name <=> b.name) }
			end

		end

	end
end
