########################################################################################################################
# Defines database common object
########################################################################################################################

require_relative 'model_error'

module Dbsketch
	module Model

		class Database_Object
			def initialize name, meaning: nil, comment: nil, dependencies: []
				dependencies = dependencies.is_a?(Array) ? dependencies : [dependencies]
				### Preconditions
				raise ArgumentError, "name is not a String" unless name.is_a? String
				raise ModelError, "name (#{name}) shall not contain special chars (apart from underscore)" if name.match(/\W/)
				raise ArgumentError, "meaning is not a String" unless nil == meaning or meaning.is_a? String
				raise ArgumentError, "comment is not a String" unless nil == comment or comment.is_a? String
				dependencies.each_with_index do |d, index|
					raise ArgumentError, "dependencies[#{index}] is not a Dbsketch::Model::Database_Object" unless d.is_a? Database_Object
				end
				###
				@name = name
				@meaning = meaning
				@comment = comment
				@dependencies = dependencies
				@order = nil
			end

			attr_reader :name, :meaning, :comment, :dependencies, :order

			def add_dependencies dependencies
				dependencies = dependencies.is_a?(Array) ? dependencies : [dependencies]
				### Preconditions
				dependencies.each_with_index do |d, index|
					raise ArgumentError, "dependencies[#{index}] is not a Dbsketch::Model::Database_Object" unless d.is_a? Database_Object
				end
				###
				@dependencies = (@dependencies.concat(dependencies)).uniq
			end

			def class_name
				self.class.name.gsub(/^Dbsketch::Model::/, '')
			end

			def inspect
				"#{self.class_name}:#{object_id} #{@name}"
			end

			def reset_order!
				@order = nil
			end

			def compute_order! trace = []
				if nil == @order
					current_trace = trace.clone
					current_trace << self
					if nil != trace.find { |t| t.object_id == self.object_id }
						raise ModelError, "circular dependencies found. Trace: #{current_trace.map { |t| "#{t.class_name} #{t.name}" }.join(" < ")}"
					end
					if @dependencies.empty?
						@order = 1
					else
						@order = @dependencies.map {|d| d.compute_order! current_trace }.max + 1
					end
				end
				@order
			end
		end

	end
end
