########################################################################################################################
# Render a column into SQL
########################################################################################################################

require_relative '../../model/abstract_column'
require_relative '../../model/column'
require_relative '../../model/computed_column'

require_relative 'type_renderer'

module Dbsketch
	module Rendering
		module Meta

			class ColumnRenderer
				def initialize type_renderer: nil, keywords: {}
					### Preconditions
					raise ArgumentError, "keywords is not a Hash" unless keywords.is_a? Hash
					###
					@keywords = {
						:column				=> "Dbsketch::Model::Column.new",
						:computed_column	=> "Dbsketch::Model::ComputedColumn.new",
						:type				=> "Dbsketch::Model::Type.new"
					}.merge keywords
					@type_renderer = (nil == type_renderer ? TypeRenderer.new(:type_keyword => @keywords[:type]) : type_renderer)
				end

				def create column
					### Preconditions
					raise ArgumentError, "column is not a Dbsketch::Model::AbstractColumn" unless column.is_a? Dbsketch::Model::AbstractColumn
					###
					if column.is_a? Dbsketch::Model::ComputedColumn
						create_computed_column column
					elsif column.is_a? Dbsketch::Model::Column
						create_column column
					else
						raise ArgumentError, "#{column.class} is an unknown subclass of Dbsketch::Model::AbstractColumn"
					end
				end

				private

				def get_semantic column
					str = ""
					str << ", :meaning => '#{column.meaning}'" if column.meaning
					str << ", :comment => '#{column.comment}'" if column.comment
					str
				end

				def create_column column
					str = "#{@keywords[:column]}('#{column.name}', #{@type_renderer.create column.type}"
					str << ", :order => #{column.order}" if nil != column.order
					str << ", :identity => true" if column.identity
					str << ", :nullable => false" if not column.nullable
					if nil != column.default
						str << ", :default => #{column.default.is_a?(String) ? "'#{column.default}'" : column.default}"
					end
					str << get_semantic(column)
					str << ")"
					str
				end

				def create_computed_column column
					str = "#{@keywords[:computed_column]}('#{column.name}', '#{column.query}'"
					str << ", :nullable => false" if not column.nullable
					str << ", :persisted => true" if column.persisted
					str << get_semantic(column)
					str << ")"
					str
				end

			end

		end
	end
end
