########################################################################################################################
# Render a column into ruby
########################################################################################################################

require_relative '../../model/function'
require_relative '../../model/procedure'

module Dbsketch
	module Rendering
		module Meta

			class OperationRenderer

				def create operation
					### Preconditions
					raise ArgumentError, "operation is not a Dbsketch::Model::Operation" unless operation.is_a? Dbsketch::Model::Operation
					###
					if operation.is_a? Dbsketch::Model::Function
						create_function operation
					elsif operation.is_a? Dbsketch::Model::Procedure
						create_procedure operation
					else
						raise ArgumentError, "#{operation.class} is an unknown subclass of Dbsketch::Model::Operation"
					end
				end

				def drop operation
					### Preconditions
					raise ArgumentError, "operation is not a Dbsketch::Model::Operation" unless operation.is_a? Dbsketch::Model::Operation
					###
					if operation.is_a? Dbsketch::Model::Function
						keyword = "Function"
					elsif operation.is_a? Dbsketch::Model::Procedure
						keyword = "Procedure"
					else
						raise ArgumentError, "#{operation.class} is an unknown subclass of Dbsketch::Model::Operation"
					end
					"#{keyword}.drop #{operation.name}"
				end

				private

				def create_function function
					arguments = function.arguments.map {|a| "\"#{a}\"" }
					arguments = function.arguments.count > 1 ? "[#{arguments.join(", ")}]" : arguments.first
					sql = "Dbsketch::Model::Function.new(\"#{function.name}\""
					sql << ", :arguments => #{arguments}" if nil != arguments
					sql << ", :returns => \"#{function.returns}\""
					sql << ", :algo => \"#{function.algo}\")"
					sql
				end

				def create_procedure procedure
					arguments = procedure.arguments.map {|a| "\"#{a}\"" }
					arguments = procedure.arguments.count > 1 ? "[#{arguments.join(", ")}]" : arguments.first
					sql = "Dbsketch::Model::Procedure.new(\"#{procedure.name}\""
					sql << ", :arguments => #{arguments}" if nil != arguments
					sql << ", :algo => \"#{procedure.algo}\")"
					sql
				end

			end

		end
	end
end
