########################################################################################################################
# Render a column into SQL
########################################################################################################################

require_relative '../../model/function'
require_relative '../../model/procedure'

module Dbsketch
	module Rendering
		module SQL

			class OperationRenderer

				def create operation
					### Preconditions
					raise ArgumentError, "operation is not a Dbsketch::Model::Operation" unless operation.is_a? Dbsketch::Model::Operation
					###
					if operation.is_a? Dbsketch::Model::Function
						create_function operation
					elsif operation.is_a? Dbsketch::Model::Procedure
						create_procedure operation
					else
						raise ArgumentError, "#{operation.class} is an unknown subclass of Dbsketch::Model::Operation"
					end
				end

				def drop operation
					### Preconditions
					raise ArgumentError, "operation is not a Dbsketch::Model::Operation" unless operation.is_a? Dbsketch::Model::Operation
					###
					if operation.is_a? Dbsketch::Model::Function
						keyword = "function"
						mssql_type = "FN"
					elsif operation.is_a? Dbsketch::Model::Procedure
						keyword = "procedure"
						mssql_type = "P"
					else
						raise ArgumentError, "#{operation.class} is an unknown subclass of Dbsketch::Model::Operation"
					end

					"if object_id('#{operation.name}', '#{mssql_type}') is not null drop #{keyword} #{operation.name}"
				end

				private

				def create_function function
					sql = "create function #{function.name} ("
					sql << "#{function.arguments.join(", ")}" if not function.arguments.empty?
					sql << ") returns #{function.returns} as\nbegin\n\t#{function.algo}\nend"
					sql
				end

				def create_procedure procedure
					sql = "create procedure #{procedure.name} "
					sql << "#{procedure.arguments.join(", ")} " if not procedure.arguments.empty?
					sql << "as\nbegin\n\t#{procedure.algo}\nend"
					sql
				end

			end

		end
	end
end
